package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.jobs.states.JobStateFactory;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.highlevel.tasks.OtherUserSshKeysTask;
import be.iminds.ilabt.jfed.highlevel.tasks.UnshareSliceTask;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleIntegerProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 11/26/2015
 * Time: 18:18
 */
public class UnshareSliceJob extends AbstractJob<Void> {
    private static final Logger LOG = LoggerFactory.getLogger(UnshareSliceJob.class);

    private final Experiment experiment;
    private final Collection<GeniUrn> usersToUnshareWith;
    private final boolean unregisterSshKey;

    private final JobStateFactory jobStateFactory;

    public UnshareSliceJob(Experiment experiment, Collection<GeniUrn> usersToUnshareWith,
                           boolean unregisterSshKey,
                           HighLevelTaskFactory hltf, TaskThread tt, JobStateFactory jobStateFactory) {
        super(String.format("Unshare slice %s with %d users", experiment.getName(), usersToUnshareWith.size()), experiment, hltf, tt);
        this.experiment = experiment;

        this.usersToUnshareWith = usersToUnshareWith;
        this.unregisterSshKey = unregisterSshKey;
        this.jobStateFactory = jobStateFactory;

    }

    @Override
    public Void execute() throws Exception {
        setAndRunState(new UnshareSliceWithUsersState());

        if (unregisterSshKey) {
            //calling update_users with empty keys will remove all users
            List<UserSpec> userSpecs = usersToUnshareWith.stream()
                    .map(user -> new UserSpec(user.toString()))
                    .collect(Collectors.toList());
            setAndRunState(jobStateFactory.createUpdateSshKeysState(
                    UnshareSliceJob.this,
                    userSpecs)
            );
        }
        return null;
    }


    public class UnshareSliceWithUsersState extends MultipleCallState<UnshareSliceTask> {
        UnshareSliceWithUsersState() {
            super(String.format("Unshare slice %s with %d users", experiment.getName(), usersToUnshareWith.size()),
                    UnshareSliceJob.this);
        }

        @Override
        public Collection<UnshareSliceTask> getTasks() {
            return usersToUnshareWith.stream()
                    .map(user -> hltf.getUnshareSliceTask(experiment.getSlice(), user))
                    .collect(Collectors.toList());
        }
    }

    public class FetchUserKeysState extends MultipleCallState<OtherUserSshKeysTask> {
        private final List<UserSpec> newUserSpecs = Collections.synchronizedList(new ArrayList<>());

        private final IntegerProperty successfulCalls = new SimpleIntegerProperty(0);
        private final IntegerProperty failedCalls = new SimpleIntegerProperty(0);

        protected FetchUserKeysState() {
            super(String.format("Fetching SSH keys of %d users", usersToUnshareWith.size()), UnshareSliceJob.this);
        }

        @Override
        public Collection<OtherUserSshKeysTask> getTasks() {
            return usersToUnshareWith.stream()
                    .map(hltf::getOtherUserSshKeys)
                    .collect(Collectors.toList());
        }

        @Override
        protected void onTaskExecutionFinished(OtherUserSshKeysTask task,
                                               TaskExecution<OtherUserSshKeysTask> taskExecution,
                                               TaskExecution.TaskState state) {
            if (taskExecution.getState() == TaskExecution.TaskState.SUCCESS) {
                newUserSpecs.add(new UserSpec(task.getUserUrn().toString(), task.getKeys()));
                successfulCalls.set(successfulCalls.get() + 1);
            } else {
                failedCalls.set(failedCalls.get() + 1);
            }
        }

        public List<UserSpec> getUserSpecs() {
            return newUserSpecs;
        }
    }
}
